'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

const prefix = '@@redux-ops/';
const CREATE = `${prefix}CREATE`;
const UPDATE = `${prefix}UPDATE`;
const DELETE = `${prefix}DELETE`;
const CLEAR = `${prefix}CLEAR`;

var action_types = /*#__PURE__*/Object.freeze({
  prefix: prefix,
  CREATE: CREATE,
  UPDATE: UPDATE,
  DELETE: DELETE,
  CLEAR: CLEAR
});

const defaultState = {};
const createOperation = (state, action) => {
    const { payload: { id }, } = action;
    if (state[id]) {
        console.error(`Warning: Cannot create new operation "${id}" because it already exists.`);
        return state;
    }
    return Object.assign({}, state, { [id]: Object.assign({}, action.payload) });
};
const updateOperation = (state, action) => {
    const { payload: { id }, } = action;
    if (!state[id]) {
        console.error(`Warning: Cannot update operation "${id}" because it doesn't exist.`);
        return state;
    }
    return Object.assign({}, state, { [id]: Object.assign({}, action.payload) });
};
const deleteOperation = (state, { payload }) => {
    const newState = Object.assign({}, state);
    const opIds = Array.isArray(payload.id) ? payload.id : [payload.id];
    opIds.forEach(opId => delete newState[opId]);
    return newState;
};
const clearOperations = () => (Object.assign({}, defaultState));
const handlers = {
    [CREATE]: createOperation,
    [UPDATE]: updateOperation,
    [DELETE]: deleteOperation,
    [CLEAR]: clearOperations,
};
const reducer = (state = defaultState, action) => {
    const handler = handlers[action.type];
    return handler ? handler(state, action) : state;
};

function constructOperation(id, status, data) {
    return { id, status, data };
}
function getOpById(ops, id) {
    return ops[id];
}
function getOpByIds(ops, ids) {
    const result = {};
    ids.forEach(id => {
        const op = ops[id];
        op && (result[id] = op);
    });
    return result;
}
function searchOps(ops, query) {
    const result = {};
    Object.keys(ops).forEach(id => {
        if (query instanceof RegExp ? !!String(id).match(query) : String(id).indexOf(query) > -1) {
            result[id] = ops[id];
        }
    });
    return result;
}

var utils = /*#__PURE__*/Object.freeze({
  constructOperation: constructOperation,
  getOpById: getOpById,
  getOpByIds: getOpByIds,
  searchOps: searchOps
});

const createOperation$1 = (id, status, data) => ({
    type: CREATE,
    payload: constructOperation(id, status, data),
});
const updateOperation$1 = (id, status, data) => ({
    type: UPDATE,
    payload: constructOperation(id, status, data),
});
const deleteOperation$1 = id => ({
    type: DELETE,
    payload: { id },
});
const clearOperations$1 = () => ({
    type: CLEAR,
});

var actions = /*#__PURE__*/Object.freeze({
  createOperation: createOperation$1,
  updateOperation: updateOperation$1,
  deleteOperation: deleteOperation$1,
  clearOperations: clearOperations$1
});

let reducerName = 'ops';
function setReducerName(name) {
    reducerName = name;
}
function getOpsState(state) {
    const subState = state[reducerName];
    if (subState === undefined) {
        throw new Error(`No reducer found with name ${reducerName}. Try to set it via setReducerName().`);
    }
    return subState;
}
function getOps(state) {
    return getOpsState(state);
}
function getOpById$1(state, id) {
    return getOpById(getOps(state), id);
}
function getOpByIds$1(state, ids) {
    return getOpByIds(getOps(state), ids);
}
function searchOps$1(state, query) {
    return searchOps(getOps(state), query);
}

var selectors = /*#__PURE__*/Object.freeze({
  setReducerName: setReducerName,
  getOpsState: getOpsState,
  getOps: getOps,
  getOpById: getOpById$1,
  getOpByIds: getOpByIds$1,
  searchOps: searchOps$1
});

class Op {
    constructor(opId) {
        this.id = '';
        this.create = (status, data) => createOperation$1(this.id, status, data);
        this.update = (status, data) => updateOperation$1(this.id, status, data);
        this.delete = () => deleteOperation$1(this.id);
        this.id = opId;
    }
    getId() {
        return this.id;
    }
}

(function (OpStatus) {
    OpStatus["Default"] = "default";
    OpStatus["Loading"] = "loading";
    OpStatus["Intermediate"] = "intermediate";
    OpStatus["Success"] = "success";
    OpStatus["Error"] = "error";
})(exports.OpStatus || (exports.OpStatus = {}));

exports.defaultState = defaultState;
exports.actionTypes = action_types;
exports.actions = actions;
exports.selectors = selectors;
exports.utils = utils;
exports.Op = Op;
exports.default = reducer;
